@@ -1,13 +1,9 @@
 from typing import List, Any
 import os
 from urllib.parse import urlparse
-from redflagbpm.BPMService import BPMService
-
 
 class Service:
-    bpm: BPMService
-
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         self.bpm = bpm
 
     def sendMail(self, msg: dict):
@@ -33,13 +29,34 @@ class Service:
         return self.bpm.call("EBHBPMService.sendMail", msg)
 
     def execute(self, script: str, context: dict):
+        """
+        Ejecuta un script en la BPM y devuelve el resultado. Útil, por ejemplo, para ejecutar
+        una plantilla VTL.
+
+        Por ejemplo:
+
+        plantilla_procesada=bpm.execute("MI_PLANTILLA_VTL",{"nombre":"Juan","edad":30})
+
+        :param script: el id del script
+        :param context: los datos que le pasamos al script como variables
+        :return: el resultado
+        """
         return self.bpm.call("EBHBPMService.execute", body={
             "script": script,
             "context": context
         })
 
     def notifyUser(self, user: str, title: str, description: str, target: str = None, sound: bool = False):
-        return self.bpm.call("EBHBPMService.notifyUser", body={
+        """
+        Emite una notificación a un usuario logueado en la BPM.
+
+        :param user: el nombre del usuario
+        :param title: el título de la notificación
+        :param description: la descripción de la notificación
+        :param target: destino de la notificación (cuando hace clic se puede redirigir a otra sección de la BPM o url)
+        :param sound: indica si e debe reproducir o no sonido
+        """
+        self.bpm.call("EBHBPMService.notifyUser", body={
             "user": user,
             "title": title,
             "description": description,
@@ -47,7 +64,16 @@ class Service:
             "sound": sound})
 
     def notifyGroup(self, group: str, title: str, description: str, target: str = None, sound: bool = False):
-        return self.bpm.call("EBHBPMService.notifyGroup", body={
+        """
+        Emite una notificación a un usuario logueado en la BPM.
+
+        :param user: el nombre del usuario
+        :param title: el título de la notificación
+        :param description: la descripción de la notificación
+        :param target: destino de la notificación (cuando hace clic se puede redirigir a otra sección de la BPM o url)
+        :param sound: indica si e debe reproducir o no sonido
+        """
+        self.bpm.call("EBHBPMService.notifyGroup", body={
             "group": group,
             "title": title,
             "description": description,
@@ -60,6 +86,9 @@ class Service:
     def now(self):
         return self.bpm.call("EBHBPMService.now", body={})
 
+    def nextBusinessKey(self, processKey: str):
+        return self.bpm.call("EBHBPMService.now", body={"processKey": processKey})
+
     def today(self):
         return self.bpm.call("EBHBPMService.today", body={})
 
@@ -107,11 +136,10 @@ class ContextJson(object):
         return json_value
 
 class Context(object):
-    bpm: BPMService
     input: ContextInput
     json: ContextJson
 
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         super(Context,self).__setattr__('bpm', bpm)
         super(Context, self).__setattr__('input', ContextInput())
         super(Context, self).__setattr__('json', ContextJson())
@@ -126,7 +154,10 @@ class Context(object):
 
     def __getattr__(self, item):
         try:
-            return self.getValue(item)
+            if item in dir(self):
+                super(Context, self).__getattribute__(item)
+            else:
+                return self.getValue(item)
         except ValueError:
             raise NameError(item) from None
 
@@ -173,9 +204,7 @@ class Context(object):
 
 
 class ResourceService:
-    bpm: BPMService
-
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         self.bpm = bpm
 
     def accessFile(self, path: str):
@@ -202,9 +231,7 @@ class ResourceService:
 
 
 class DocumentService:
-    bpm: BPMService
-
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         self.bpm = bpm
 
     def registerCollection(self, schema: dict):
@@ -268,9 +295,7 @@ class DocumentService:
 
 
 class RuntimeService:
-    bpm: BPMService
-
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         self.bpm = bpm
 
     def startProcessInstanceByKey(self, processDefinitionKey: str, businessKey: str, variables: dict):
@@ -340,9 +365,7 @@ class RuntimeService:
 
 
 class ExecutionService:
-    bpm: BPMService
-
-    def __init__(self, bpm: BPMService):
+    def __init__(self, bpm):
         self.bpm = bpm
 
     def updateBusinessKey(self, businessKey: str):
