@@ -6,7 +6,7 @@ import redflagbpm
 bpm=redflagbpm.BPMService()
 
 def holaMundo(msg):
-    bpm.setAddress(msg)
+    bpm.__setAddress(msg)
     bpm.service.notifyUser("lbessone","Hola!","Desde python3!")
     bpm.reply("Hola Mundo!")
 
